BeginPackage["NDSolveProgressBar`", "Interface`"]

NDSolveProgressBar::usage = "NDSolveProgressBar works just like NDSolve, but it displays a progress bar indicating the advance of the iterations."

Begin["`Private`"]

Bar = Component["Wolfram/Base/Frame", {
        "title" -> "Progress Dialog",
        "resizable" -> False,
        PlaceComponent[
          Component["Wolfram/Base/Panel", {
              "layout" -> ToClass["java.awt.LayoutManager"],
              PlaceComponent[
                  Component["Wolfram/Base/Label", {
                    "text" -> "Percent complete:",
                    Call["setBounds", 20, 10, 200, 20]}, 
                     Name -> "label"],
                  Component["Wolfram/Base/Label", {
                    "text" -> "Steps",
                    Call["setBounds", 200, 10, 230, 20]}, 
                     Name -> "steps"],
                 Component["Wolfram/Base/ProgressBar", {
                    "minimum" -> 0,
                    "maximum" -> 100,
                    Call["setBounds", 20, 40, 260, 30]},
                     Name -> "bar"]  ],
               
              Call["setPreferredSize", 
                Component["Wolfram/Base/Dimension", 
                  Arguments -> {300, 110}]]}  ]
              ],
        Call["setSize", 300, 110],
        Call["setLocation", 0, 0]},
      Name -> "frame"];

SetUpMonitor[mon_, b_] := RuleDelayed[mon, b]

SetUpMonitor[mon_, b_, Hold[s_]] := RuleDelayed[mon, CompoundExpression[b, s]]

NDSolveProgressBar[eqns_, dvars_, {ivar_, start_, end_}, space:{_Symbol, __}..., opts___?OptionQ] := 
Module[{L, expr, smon, res, symbar, bar, states, state,inc, s = 0, emon, ev = 0},
    SetAttributes[inc, HoldAll];
    L = end - start;
    expr = ce[
        symbar@SetPropertyValue[{"bar", "value"},Ceiling[(ivar - start) 100./L]],
        symbar@SetPropertyValue[{"steps", "text"},ts[inc[s]]]
    ];
    smon = Cases[{opts}, Alternatives[Rule, RuleDelayed][StepMonitor, value_]];
    If[Length[smon] > 0,
        smon = SetUpMonitor[StepMonitor, expr, HeldPart[smon, 1, 2]],
        smon = SetUpMonitor[StepMonitor, expr]
    ];
    smon = smon /. {ce->CompoundExpression, symbar->bar, inc->PreIncrement, ts->Function[StringJoin[ToString[#], " steps"]]};
    bar = InterfaceRun[Bar];
    bar@SetPropertyValue[{"label", "text"}, "Processing equations..."];
    states = NDSolve`ProcessEquations[eqns, dvars, {ivar, start, end}, space, smon, opts];
    res = MapIndexed[Function[
        bar@SetPropertyValue[{"frame", "title"}, StringJoin["Solving system", ToString[First[#2]]," ..."]];
        state = #1;
        bar@SetPropertyValue[{"label", "text"}, "Iterating..."];
        NDSolve`Iterate[state, {start, end}];
        bar@SetPropertyValue[{"label", "text"}, "Processing solution functions..."];
        NDSolve`ProcessSolutions[state]],
        states];
    ReleaseInterfaceObject[bar];
    res]

End[]

EndPackage[]