(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     15428,        438]*)
(*NotebookOutlinePosition[     16301,        469]*)
(*  CellTagsIndexPosition[     16226,        463]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell[CellGroupData[{

Cell["Array valued variables", "Section"],

Cell[TextData[{
  "The functions where this is currently supported are ",
  StyleBox["FindRoo", "MR"],
  "t, ",
  StyleBox["FindMaximum", "MR"],
  ", ",
  StyleBox["FindMinimum", "MR"],
  ", and ",
  StyleBox["NDSolve", "MR"],
  ".  Something all of these functions have in common is that they use \
initial conditions or a starting point.  When the value of the starting point \
is an array, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " interprets the variable to be array-valued.  Some examples will point out \
this idea much more clearly."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["Here is a matrix and a vector", "MathCaption"],

Cell[BoxData[
    RowBox[{
      RowBox[{"A", " ", "=", " ", 
        RowBox[{"(", GridBox[{
              {"1", "2", "3"},
              {"4", "5", "6"},
              {"7", "8", "9"}
              }], ")"}]}], ";", \(b\  = \ {14, 32, 50}\), ";"}]], "Input",
  CellLabel->"In[10]:="],

Cell["This solves a system of linear equations.", "MathCaption"],

Cell[BoxData[
    \(FindRoot[A . x\  \[Equal] \ b, {x, {1, 1, 1}}]\)], "Input",
  CellLabel->"In[11]:="],

Cell["This finds an eigenvalue and the corresponding eigenvector.", \
"MathCaption"],

Cell[BoxData[
    \(FindRoot[{A . x\  \[Equal] \ \[Lambda]\ x, \ 
        x . x\  \[Equal] \ 1}, {{\[Lambda], \ 
          1}, \ {x, {1, 1, 1}}}]\)], "Input",
  CellLabel->"In[12]:="],

Cell["\<\
The above two problems could, of course have been expressed through scalar \
variables, but the vector notation mathematically expresses the idea of the \
problem quite well.\
\>", "Text"],

Cell["This expresses the equations in terms of scalar variables", \
"MathCaption"],

Cell[BoxData[{
    \(\(xx\  = \ {x, y, z};\)\), "\[IndentingNewLine]", 
    \(Join[
      Thread[A . xx\  \[Equal] \ \[Lambda]\ xx], {xx . xx\  \[Equal] \ 
          1}]\)}], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \(FindRoot[%, \ {{\[Lambda], 1}, {x, 1}, {y, 1}, {z, 1}}]\)], "Input",
  CellLabel->"In[15]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["\<\
In differential equations, many problems are most naturally expressed in \
terms of vectors.\
\>", "Text"],

Cell[TextData[{
  "Here is a program which computes intrinsic curves using the fundamental \
theorem of space curves, by uniquely (except for rigid motions) defining a \
space curve based knowing curvature ",
  Cell[BoxData[
      \(TraditionalForm\`\[Kappa][t]\)]],
  " and torsion ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau][t]\)]],
  ". This is linear time varying system, of dimension 12. "
}], "MathCaption"],

Cell[BoxData[{
    \(\(IntrinsicCurve3D[\ {\[Kappa]_, \ \[Tau]_}, {s_, \ smin_, \ 
            smax_}, \ {s0_?NumericQ, \ x0_?VectorQ, \ t0_?VectorQ, \ 
            n0_?VectorQ}, \ 
          opts___?OptionQ]\  := \[IndentingNewLine]Module[\ {x, \ t, \ n, \ 
            b, \ s, \ c}, \[IndentingNewLine]points\  = \ 
            Last[Reap[\[IndentingNewLine]NDSolve[\[IndentingNewLine]{\
\[IndentingNewLine]\(x'\)[s]\  \[Equal] \ 
                      t[s], \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ x[s0] \[Equal] \ 
                      x0, \[IndentingNewLine]\(t'\)[
                        s]\  \[Equal] \ \[Kappa]\ n[
                          s], \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ t[s0] \[Equal] \ 
                      t0, \[IndentingNewLine]\(n'\)[
                        s]\  \[Equal] \ \[Tau]\ b[s] - \ \[Kappa]\ t[
                            s], \ \ \ \ \ \ \ \ \ \ \ n[
                        s0] \[Equal] \ \ n0, \[IndentingNewLine]\(b'\)[
                        s]\  \[Equal] \ \(-\[Tau]\)\ n[
                          s], \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ b[s0] \[Equal] \ Cross[\ t0, \ 
                        n0]\[IndentingNewLine]}, \[IndentingNewLine]{}, \ \
\[IndentingNewLine]{s, \ smin, \ 
                    smax}, \[IndentingNewLine]StepMonitor \[RuleDelayed] 
                    Sow[x[s]], \ 
                  opts\[IndentingNewLine]]]]; \[IndentingNewLine]Show[
            Graphics3D[
              Line\  @@ \ 
                points]]\[IndentingNewLine]];\)\), "\[IndentingNewLine]", 
    \(\(Off[NDSolve::noout];\)\)}], "Input",
  CellLabel->"In[19]:="],

Cell[TextData[{
  "Note that the NDSolve::noout message is just a warning from ",
  StyleBox["NDSolve", "MR"],
  " that no functions are being prepared for output.  The program turns off \
the message since it is taking its output from the ",
  StyleBox["StepMonitor", "MR"],
  "."
}], "Text"],

Cell["This gives default values for where to start the curve.", "MathCaption"],

Cell[BoxData[
    \(\(IntrinsicCurve3D[\ {\[Kappa]_, \ \[Tau]_}, \ {s_, \ smin_, \ 
            smax_}, \ 
          opts___?OptionQ\ ]\  := \ \[IndentingNewLine]IntrinsicCurve3D[{\
\[Kappa], \ \[Tau]}, \ {s, \ smin, \ 
            smax}, \ {0, \ {0, 0, 0}, \ {1, 0, 0}, \ {0, 1, 0}}, \ 
          opts\ ];\)\)], "Input",
  CellLabel->"In[17]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["\<\
Here is an example of a curve defined by curvature \[Kappa] = |s| and torsion \
\[Tau] = .3\
\>", "MathCaption"],

Cell[BoxData[
    \(\(IntrinsicCurve3D[\ {Abs[s], \ 0.3}, \ {s, \(-10\), \ 
          10}];\)\)], "Input",
  CellLabel->"In[21]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[GridBox[{
        {
          ButtonBox[
            StyleBox["\[FirstPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageFirst"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"First Slide"], 
          ButtonBox[
            StyleBox["\[LeftPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPagePrevious"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Previous Slide"], 
          ButtonBox[
            StyleBox["\[RightPointer]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageNext"]}],
            ButtonStyle->"SlideHyperlink",
            ButtonNote->"Next Slide"], 
          ButtonBox[
            StyleBox["\[LastPage]",
              "SR"],
            ButtonFunction:>FrontEndExecute[ {
                FrontEndToken[ 
                  FrontEnd`SelectedNotebook[ ], "ScrollPageLast"]}],
            ButtonStyle->"SlideHyperlink"], " ", 
          ButtonBox[
            StyleBox[
              RowBox[{
                CounterBox["SlideShowNavigationBar"], \(\(\ \)\(of\)\(\ \)\), 
                
                CounterBox["SlideShowNavigationBar", "LastSlide"]}],
              "SR"],
            ButtonFrame->"None"]}
        }]]]], "SlideShowNavigationBar"],

Cell["\<\
Here is an example of a curve defined with constant curvature \[Kappa] = 1.3 \
and torsion \[Tau] = Sin[s].\
\>", "MathCaption",
  CellTags->"LastSlide"],

Cell[BoxData[
    \(\(IntrinsicCurve3D[\ { .9, \ 0.8\ Sin[\ s]}, \ {s, 0, \ 
          10\ \[Pi]}];\)\)], "Input",
  CellLabel->"In[22]:="]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
ScreenStyleEnvironment->"SlideShow",
WindowSize->{1016, 696},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
Magnification->1.5,
StyleDefinitions -> "New In 5.0 Style.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "LastSlide"->{
    Cell[15107, 426, 163, 4, 64, "MathCaption",
      CellTags->"LastSlide"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"LastSlide", 16122, 456}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 1594, 41, 20, "SlideShowNavigationBar"],

Cell[CellGroupData[{
Cell[3373, 96, 41, 0, 76, "Section"],
Cell[3417, 98, 569, 16, 114, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4023, 119, 1594, 41, 20, "SlideShowNavigationBar"],
Cell[5620, 162, 52, 0, 43, "MathCaption"],
Cell[5675, 164, 284, 8, 92, "Input"],
Cell[5962, 174, 64, 0, 43, "MathCaption"],
Cell[6029, 176, 104, 2, 51, "Input"],
Cell[6136, 180, 84, 1, 43, "MathCaption"],
Cell[6223, 183, 183, 4, 51, "Input"],
Cell[6409, 189, 198, 4, 62, "Text"],
Cell[6610, 195, 82, 1, 43, "MathCaption"],
Cell[6695, 198, 208, 5, 77, "Input"],
Cell[6906, 205, 113, 2, 40, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7056, 212, 1594, 41, 28, "SlideShowNavigationBar"],
Cell[8653, 255, 116, 3, 34, "Text"],
Cell[8772, 260, 420, 10, 108, "MathCaption"],
Cell[9195, 272, 1662, 30, 482, "Input"],
Cell[10860, 304, 293, 7, 80, "Text"],
Cell[11156, 313, 78, 0, 64, "MathCaption"],
Cell[11237, 315, 346, 7, 118, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11620, 327, 1594, 41, 28, "SlideShowNavigationBar"],
Cell[13217, 370, 122, 3, 64, "MathCaption"],
Cell[13342, 375, 131, 3, 40, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13510, 383, 1594, 41, 28, "SlideShowNavigationBar"],
Cell[15107, 426, 163, 4, 64, "MathCaption",
  CellTags->"LastSlide"],
Cell[15273, 432, 139, 3, 40, "Input"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

